@@ -13,12 +13,15 @@
 
 module Language.Preprocessor.Cpphs.ReadFirst
   ( readFirst
+  , readFileUTF8
+  , writeFileUTF8
   ) where
 
-import System.IO        (hPutStrLn, stderr)
+import System.IO
 import System.Directory (doesFileExist)
-import Data.List      (intersperse)
-import Control.Monad     (when)
+import Data.List        (intersperse)
+import Control.Exception as E
+import Control.Monad    (when)
 import Language.Preprocessor.Cpphs.Position  (Posn,directory,cleanPath)
 
 -- | Attempt to read the given file from any location within the search path.
@@ -51,5 +54,20 @@ readFirst name demand path warn =
         let file = cleanPath p++'/':cleanPath name
         ok <- doesFileExist file
         if not ok then try name ps
-          else do content <- readFile file
+          else do content <- readFileUTF8 file
                   return (file,content)
+
+readFileUTF8 :: FilePath -> IO String
+readFileUTF8 file = do
+    h <- openFile file ReadMode
+    (do utf8r <- mkTextEncoding "UTF-8//ROUNDTRIP"
+        hSetEncoding h utf8r
+        hGetContents h) `E.onException` (hClose h)
+
+writeFileUTF8 :: FilePath -> String -> IO ()
+writeFileUTF8 f txt = withFile f WriteMode $ \hdl->
+                          do utf8r <- mkTextEncoding "UTF-8//ROUNDTRIP"
+                             hSetEncoding hdl utf8r
+                             hPutStr hdl txt
+                          `E.onException` (hClose hdl)
+
